CREATE TABLE attemper_arg (
  ARG_NAME VARCHAR(64) NOT NULL,
  ARG_TYPE INTEGER NOT NULL,
  ARG_VALUE VARCHAR(2000),
  GENERIC_TYPE INTEGER,
  ATTRIBUTE VARCHAR(2000),
  REMARK VARCHAR(255)  ,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (ARG_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_calendar */

CREATE TABLE attemper_calendar (
  CALENDAR_NAME VARCHAR(64) NOT NULL,
  DISPLAY_NAME VARCHAR(255),
  PRIMARY KEY (CALENDAR_NAME)
) ;

/*Table structure for table attemper_calendar_day */

CREATE TABLE attemper_calendar_day (
  CALENDAR_NAME VARCHAR(64)  NOT NULL,
  DAY_NUM INTEGER NOT NULL,
  REMARK VARCHAR(255)  ,
  PRIMARY KEY (CALENDAR_NAME,DAY_NUM)
) ;

/*Table structure for table attemper_condition */

CREATE TABLE attemper_condition (
  CONDITION_NAME VARCHAR(64)  NOT NULL,
  CONDITION_TYPE INTEGER NOT NULL,
  CONTENT VARCHAR(2000)  NOT NULL,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (CONDITION_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_datasource */

CREATE TABLE attemper_datasource (
  DB_NAME VARCHAR(64) NOT NULL,
  DRIVER_CLASS_NAME VARCHAR(64) NOT NULL,
  JDBC_URL VARCHAR(255) NOT NULL,
  USER_NAME VARCHAR(255),
  PASSWORD VARCHAR(255),
  ATTRIBUTE VARCHAR(255),
  TENANT_ID VARCHAR(64) NOT NULL,
  PRIMARY KEY (DB_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_delay_job */

CREATE TABLE attemper_delay_job (
  ID VARCHAR(64) NOT NULL,
  JOB_NAME VARCHAR(64) NOT NULL,
  STATUS INTEGER,
  REQUEST_TIME BIGINT,
  TENANT_ID VARCHAR(255) NOT NULL,
  PRIMARY KEY (ID)
);

/*Table structure for table attemper_execution */

CREATE TABLE attemper_execution (
  ID VARCHAR(64)  NOT NULL,
  PROC_INST_ID VARCHAR(64)  ,
  ROOT_PROC_INST_ID VARCHAR(64)  ,
  SUPER_PROC_INST_ID VARCHAR(64)  ,
  JOB_NAME VARCHAR(64)  ,
  TRIGGER_NAME VARCHAR(64)  ,
  PROC_DEF_ID VARCHAR(64)  ,
  START_TIME BIGINT ,
  END_TIME BIGINT ,
  DURATION BIGINT ,
  STATUS INTEGER,
  CODE INTEGER,
  MSG TEXT,
  PARENT_ID VARCHAR(64)  ,
  SCHEDULER_URI VARCHAR(255)  ,
  EXECUTOR_URI VARCHAR(255)  ,
  TENANT_ID VARCHAR(64)  ,
  PRIMARY KEY (ID),
  UNIQUE (PROC_INST_ID)
) ;

create index IDX_EXECUTION_JOB_NAME on attemper_execution(JOB_NAME);
create index IDX_EXECUTION_TENANT_ID on attemper_execution(TENANT_ID);

/*Table structure for table attemper_gist */

CREATE TABLE attemper_gist (
  GIST_NAME VARCHAR(64)  NOT NULL,
  CREATE_TIME BIGINT NOT NULL,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (GIST_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_gist_info */

CREATE TABLE attemper_gist_info (
  ID VARCHAR(64) NOT NULL,
  GIST_NAME VARCHAR(64) NOT NULL,
  VERSION VARCHAR(64) NOT NULL,
  CONTENT TEXT ,
  UPDATE_TIME BIGINT NOT NULL,
  TENANT_ID VARCHAR(64) NOT NULL,
  PRIMARY KEY (ID)
) ;

/*Table structure for table attemper_instance */

CREATE TABLE attemper_instance (
  ID VARCHAR(64)  NOT NULL,
  PROC_INST_ID VARCHAR(64)  ,
  ROOT_PROC_INST_ID VARCHAR(64)  ,
  SUPER_PROC_INST_ID VARCHAR(64)  ,
  JOB_NAME VARCHAR(64),
  TRIGGER_NAME VARCHAR(64),
  PROC_DEF_ID VARCHAR(64),
  START_TIME BIGINT,
  END_TIME BIGINT,
  DURATION BIGINT,
  STATUS INTEGER,
  CODE INTEGER,
  MSG TEXT ,
  PARENT_ID VARCHAR(64)  ,
  SCHEDULER_URI VARCHAR(255)  ,
  EXECUTOR_URI VARCHAR(255)  ,
  TENANT_ID VARCHAR(64),
  PRIMARY KEY (ID),
  UNIQUE (PROC_INST_ID)
) ;

create index IDX_INSTANCE_JOB_NAME on attemper_instance(JOB_NAME);
create index IDX_INSTANCE_TENANT_ID on attemper_instance(TENANT_ID);

/*Table structure for table attemper_instance_act */

CREATE TABLE attemper_instance_act (
  ID VARCHAR(64) NOT NULL,
  ACT_INST_ID VARCHAR(64),
  PARENT_ACT_INST_ID VARCHAR(64),
  PROC_INST_ID VARCHAR(64),
  ROOT_PROC_INST_ID VARCHAR(64),
  ACT_ID VARCHAR(255),
  ACT_NAME VARCHAR(255),
  ACT_TYPE VARCHAR(255),
  START_TIME BIGINT,
  END_TIME BIGINT,
  DURATION BIGINT,
  STATUS INTEGER,
  LOG_KEY VARCHAR(255),
  LOG_TEXT TEXT,
  BIZ_URI VARCHAR(255),
  PRIMARY KEY (ID)
) ;

create index IDX_INSTANCE_ACT_PROC_INST_ID on attemper_instance_act(PROC_INST_ID);

/*Table structure for table attemper_job */

CREATE TABLE attemper_job (
  JOB_NAME VARCHAR(64)  NOT NULL,
  DISPLAY_NAME VARCHAR(255)  ,
  CONTENT TEXT ,
  STATUS INTEGER,
  CONCURRENT INTEGER,
  ONCE INTEGER,
  UPDATE_TIME BIGINT,
  REMARK VARCHAR(255),
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (JOB_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_job_arg */

CREATE TABLE attemper_job_arg (
  JOB_NAME VARCHAR(64)  NOT NULL,
  ARG_NAME VARCHAR(64)  NOT NULL,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (JOB_NAME,ARG_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_job_condition */

CREATE TABLE attemper_job_condition (
  JOB_NAME VARCHAR(64)  NOT NULL,
  CONDITION_NAME VARCHAR(64)  NOT NULL,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (JOB_NAME,CONDITION_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_job_project */

CREATE TABLE attemper_job_project (
  JOB_NAME VARCHAR(64)  NOT NULL,
  PROJECT_NAME VARCHAR(64)  ,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (JOB_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_project */

CREATE TABLE attemper_project (
  PROJECT_NAME VARCHAR(64)  NOT NULL,
  PARENT_PROJECT_NAME VARCHAR(64)  ,
  DISPLAY_NAME VARCHAR(255)  ,
  CONTEXT_PATH VARCHAR(255)  ,
  BIND_EXECUTOR INTEGER,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (PROJECT_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_project_executor */

CREATE TABLE attemper_project_executor (
  PROJECT_NAME VARCHAR(64) NOT NULL,
  EXECUTOR_URI VARCHAR(255) ,
  TENANT_ID VARCHAR(64) NOT NULL,
  PRIMARY KEY (PROJECT_NAME,TENANT_ID)
) ;

/*Table structure for table attemper_project_info */

CREATE TABLE attemper_project_info (
  PROJECT_NAME VARCHAR(64)  NOT NULL,
  URI VARCHAR(255) NOT NULL,
  URI_TYPE INTEGER,
  TENANT_ID VARCHAR(64)  NOT NULL,
  PRIMARY KEY (PROJECT_NAME,URI,TENANT_ID)
) ;

/*Table structure for table attemper_role */

CREATE TABLE attemper_role (
  ROLE_NAME VARCHAR(64)  NOT NULL,
  DISPLAY_NAME VARCHAR(255)  ,
  REMARK VARCHAR(255)  ,
  PRIMARY KEY (ROLE_NAME)
) ;

/*Table structure for table attemper_role_resource */

CREATE TABLE attemper_role_resource (
  ROLE_NAME VARCHAR(64) NOT NULL,
  RESOURCE_NAME VARCHAR(64) NOT NULL,
  PRIMARY KEY (ROLE_NAME,RESOURCE_NAME)
) ;

/*Table structure for table attemper_role_tenant */

CREATE TABLE attemper_role_tenant (
  ROLE_NAME VARCHAR(64) NOT NULL,
  USER_NAME VARCHAR(64) NOT NULL,
  PRIMARY KEY (ROLE_NAME,USER_NAME)
) ;

/*Table structure for table attemper_tenant */

CREATE TABLE attemper_tenant (
  USER_NAME VARCHAR(64)  NOT NULL,
  DISPLAY_NAME VARCHAR(255)  NOT NULL,
  PASSWORD VARCHAR(255)  NOT NULL,
  EMAIL VARCHAR(255)  ,
  MOBILE VARCHAR(255)  ,
  STATUS INTEGER NOT NULL,
  SUPER_ADMIN INTEGER,
  SEND_CONFIG VARCHAR(64)  ,
  PRIMARY KEY (USER_NAME)
) ;
